<?php

namespace Drupal\eav_field\Routing;

use Drupal\Core\Entity\EntityFieldManagerInterface;
use Symfony\Component\Routing\Route;
use Drupal\eav_field\Form\EavEntityAttributesForm;

class EavFieldRoutes {

  /**
   * Dynamic module routes.
   */
  public function routes() {
    $routes = [];

    $entity_type_manager = \Drupal::entityTypeManager();
    $entity_field_manger = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $entity_field_manger */
    $eav_fields = $entity_field_manger->getFieldMapByFieldType('eav');

    foreach (array_keys($eav_fields) as $entity_type_id) {
      $entity_type = $entity_type_manager->getDefinition($entity_type_id);

      $route = [
        'path' => $entity_type->getLinkTemplate('edit-form') . '/eav-attributes',
        'defaults' => [
          '_form' => EavEntityAttributesForm::class,
        ],
        // @TODO Replace entity edit permission
        'requiremets' => [
          '_permission' => 'administer eav attributes',
        ],
        'options' => [
          '_admin_route' => TRUE,
          // Commerce product edit-form route needs loaded product entity
          'parameters' => [
            $entity_type_id => [
              'type' => 'entity:' . $entity_type_id,
            ],
          ],
        ],
      ];

      $routes["entity.$entity_type_id.edit_eav_attributes"] = new Route($route['path'], $route['defaults'], $route['requiremets'], $route['options']);
    }

    return $routes;
  }

}
