<?php

namespace Drupal\Tests\eav_field\Traits;

use Drupal\Core\Entity\EntityInterface;
use Drupal\eav_field\Plugin\Field\FieldType\EavItemList;

trait EavFieldTestTrait {

  /**
   * Return last added attribute id.
   */
  public function getLastAddedEavAttributeId() {
    $query = \Drupal::database()->select('eav_attribute', 'a');
    $query->addExpression('MAX(a.aid)');
    return $query->execute()->fetchField();
  }

  /**
   * Return entity eav field items.
   */
  public function getEavFieldItems(EntityInterface $entity, string $field_name = 'field_eav'): ?EavItemList {
    return $entity->get($field_name);
  }

  /**
   * Reset entity storage caches.
   */
  public function resetEntityStorageCaches() {
    $entity_type_manager = \Drupal::entityTypeManager(); // Dont use $this->container->get('entity_type.manager'), it's not work
    foreach ($entity_type_manager->getDefinitions() as $id => $definition) {
      $entity_type_manager->getStorage($id)->resetCache();
    }
  }

  /**
   * Check system message text.
   */
  public function seeSystemMessage($message) {
    $this->assertSession()->elementTextContains('css', '.messages', $message);
  }

}
